#include <stdlib.h>
#include <stdio.h>
#include "argsparse.h"
#include "fatals.h"

// integer functions
int getInt(char * arg) {
  char c;
  if(arg==NULL)
    fatal("Argument is nonexistent.\n");
  if(arg[0]=='-') c = arg[1];
  else c = arg[0];

  if(c>'9'||c<'0') 
    fatal("Argument %s is not an integer.\n",arg);

  return atoi(arg);
}

int getIntOnRangeInclusive(char * arg, int low, int high) {
  int x = getInt(arg);
  if(x<low||x>high) fatal("Argument %s is not on range [%i,%i]\n",arg,low,high);
  return x;
}

int getIntOnRangeExclusive(char * arg, int low, int high) {
  int x = getInt(arg);
  if(x<=low||x>=high) fatal("Argument %s is not on range (%i,%i)\n",arg,low,high);
  return x;
}

// double functions
double getDouble(char * arg) {
  char c;
  if(arg==NULL)
    fatal("Argument is nonexistent.\n");
  if(arg[0]=='-') {
    if(arg[1]=='.') {
      c = arg[2];
    } else {
      c = arg[1];
    }
  } else {
    if(arg[0]=='.') {
      c = arg[1];
    } else {
      c = arg[0];
    }
  }

  if(c>'9'||c<'0')
    fatal("Argument %s is not a double\n",arg);

  return atof(arg);  
}

double getDoubleOnRangeInclusive(char * arg, double low, double high) {
  double x = getDouble(arg);
  if(x<low||x>high) {
    char bufflow[16];
    sprintf(bufflow,"%7.7f",low);
    char buffhigh[16];
    sprintf(buffhigh,"%7.7f",high);
    fatal("Double representation of argument %s is not on range [%s,%s]\n",arg,bufflow,buffhigh);
  }
  return x;
}

double getDoubleOnRangeExclusive(char * arg, double low, double high) {
  double x = getDouble(arg);
  if(x<=low||x>=high) {
    char bufflow[16];
    sprintf(bufflow,"%7.7f",low);
    char buffhigh[16];
    sprintf(buffhigh,"%7.7f",high);
    fatal("Double representation of argument %s is not on range (%s,%s)\n",arg,bufflow,buffhigh);
  }
  return x;
}



