#include "progbar.h"
#include <string.h>
#include <sys/ioctl.h>

const char * const green = "\033[1;40;32m";
const char * const blue = "\033[1;40;34m";
const char * const red = "\033[1;40;31m";
const char * const yellow = "\033[1;40;33m";
const char * const white = "\033[1;40;37m";
const char * const default_color = "\033[0;40;37m";

ProgressBar::ProgressBar() {
  myStream = NULL;
  myPos = 0;
  drawn = false;
  show_percent = true;

  // get current width of the display
  struct winsize ws;
  if(ioctl(fileno(stdout),TIOCGWINSZ,&ws) != -1 && 0 < ws.ws_col)
    windowSize = ws.ws_col - 1;
  else 
    windowSize = 80 -1; // best guess!

}

ProgressBar::~ProgressBar() {
  if(drawn) eraseBar();
}

int ProgressBar::setPos(int pos) {
  if(pos<0||pos>100) return 1;
  myPos = pos;
  return 0;
}
  
int ProgressBar::setStream(FILE * stream) {
  myStream = stream;
  drawn = false;
  if(fileno(stream)==-1) return 1;
  return 0;
}

int ProgressBar::updateBar() {
  int x = 0;
  if(drawn) x = eraseBar();
  if(x) return x;
  return drawBar();
}

void ProgressBar::showPercent(bool ok) {
  show_percent = ok;  
}

int ProgressBar::drawBar() {
  myBarLen = 0;
  int i;
 

  if(fileno(myStream)==-1) {
    return 1;
  }

  const char * color = 
    (myStream!=stdout && myStream!=stderr) ? "" :
    (myPos<25) ? red : 
    (myPos<100) ? blue : green;

  // draw the percentage sign
  if(show_percent) {
    myBarLen+=fprintf(myStream,"%s%3i%s ",white,myPos,"%");
  }

  myBarLen-=strlen(color);

  // draw the bar itself
  if(myStream!=stdout&&myStream!=stderr)
    fprintf(myStream,"[");
  else 
    fprintf(myStream,"%s[%s",yellow,color);
    
  myBarLen++;
    

  double percent = myPos / 100.0;
  int nDrawTotal = windowSize - 4 - 2;
  int nBars = (int) (percent * nDrawTotal);

  // draw bars
  for(i=0;i<nBars;i++) fprintf(myStream,"=");
  myBarLen+=nBars;

  // draw blank space
  for(i;i<nDrawTotal;i++) myBarLen+=fprintf(myStream," ");

  // terminate the bloody thing
  if(myStream!=stdout&&myStream!=stderr)
    fprintf(myStream,"]");
  else 
    fprintf(myStream,"%s]%s",yellow,color);

  myBarLen++;

  if(myStream==stdout||myStream==stderr) 
    fprintf(myStream,"%s",default_color);

  drawn = true;

  if(myBarLen-1!=windowSize) {
    fprintf(stderr,"myBarLen = %i\nwindowSize = %i\n",myBarLen-1,windowSize);
    return 1;
  }

  return 0;
}

int ProgressBar::eraseBar() {
  int i;
  
  if(fileno(myStream)==-1) {
    return 1;
  }

  if(myStream==stdout||myStream==stderr)
    for(i=0;i<myBarLen;i++) fprintf(myStream,"\b");
  else
    fseek(myStream,-myBarLen,SEEK_CUR);
  

  drawn = false;
  return 0;
}


